from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.contacts.utils import safe_html
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest import TestCase


class TestSafeHtml(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_returns_safe_html(self):
        self.assertEquals(
            'Ch\xc3\xa4ck',
            safe_html('<script>bad code</script>Ch\xc3\xa4ck'))

    def test_returns_empty_string_if_text_is_None(self):
        self.assertEquals('', safe_html(None))

    def test_returns_safe_html_with_unicode(self):
        self.assertEquals(
            'Ch\xc3\xa4ck',
            safe_html(u'<script>bad</script>Ch\xe4ck'))

    def test_transform_nl_to_br(self):
        self.assertEquals(
            'Ch\xc3\xa4ck<br />',
            safe_html('<script>bad</script>Ch\xc3\xa4ck\n'))



from ftw.upgrade import UpgradeStep
from Products.CMFPlone.utils import getFSVersionTuple

PLONE5 = getFSVersionTuple() >= (5, 0)


class RemoveObosleteBehaviourInPlone5(UpgradeStep):
    """Remove oboslete behaviour in Plone 5.
    """

    def __call__(self):
        if PLONE5:
            self.install_upgrade_profile()



from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.geo.interfaces import IGeocodableLocation
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest import TestCase
from zope.component import queryAdapter


class TestLocationString(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_none_if_no_address_zip_and_city(self):
        contact = create(Builder('contact'))

        location_adapter = queryAdapter(contact, IGeocodableLocation)

        self.assertEqual(
            None, location_adapter.getLocationString(),
            """If there is no address, no zip_code and no city
            on the contact, it is not possible to do a meaningful
            geocode lookup. So it should return None""")

    def test_full_string_if_address_zip_and_city_is_set(self):
        contact = create(Builder('contact').having(
            address=u"Ch\xfcckstreet 12  ",
            postal_code=u"1234  ",
            city=u"New York  "))

        location_adapter = queryAdapter(contact, IGeocodableLocation)

        self.assertEqual(
            u"Ch\xfcckstreet 12, 1234, New York, Schweiz",
            location_adapter.getLocationString(),
            "The order should be: street, zip, city, country")

    def test_location_string_if_some_info_is_missing(self):
        contact = create(Builder('contact').having(
            address=u"   ",
            city=u"New York"))

        location_adapter = queryAdapter(contact, IGeocodableLocation)

        self.assertEqual(
            u"New York, Schweiz",
            location_adapter.getLocationString(),
            """The adapter should concatenate only the fields with a value""")

    def test_remove_postfach_if_exists_in_address(self):
        contact = create(Builder('contact').having(
            address=u"Ch\xfcckstreet 12\rPostfach",
            city=u"New York"))

        location_adapter = queryAdapter(contact, IGeocodableLocation)

        self.assertEqual(
            u"Ch\xfcckstreet 12, New York, Schweiz",
            location_adapter.getLocationString(),
            """The adapter should concatenate only the fields with a value""")



from Acquisition import aq_inner, aq_parent
from ftw.contacts import _
from ftw.contacts import IS_PLONE_5
from ftw.contacts.interfaces import IMemberBlock
from ftw.contacts.utils import get_backreferences
from ftw.contacts.utils import safe_html
from ftw.contacts.utils import portrait_img_tag
from plone import api
from Products.Five.browser import BrowserView
from z3c.form.interfaces import HIDDEN_MODE


if IS_PLONE_5:
    from plone.app.content.browser.actions import RenameForm as PloneRenameForm

# Check for ftw.geo
try:
    from collective.geo.geographer.interfaces import IGeoreferenceable
    from collective.geo.mapwidget.browser.widget import MapWidget

except ImportError:
    HAS_GEO_EXTRA = False
else:
    HAS_GEO_EXTRA = True


class ContactView(BrowserView):
    """Contact view
    """
    def get_memberships(self):
        memberships = get_backreferences(self.context, IMemberBlock)
        # Sort the memberships alphabetically by the title of their container before returning.
        return sorted(memberships, key=lambda membership: aq_parent(aq_inner(membership)).Title().lower())

    def safe_html(self, text):
        return safe_html(text)

    def show_map(self):
        """
        """
        return HAS_GEO_EXTRA and IGeoreferenceable.providedBy(self.context) and not self.context.hide_map

    def get_address_map(self):
        address_map = MapWidget(self, self.request, self.context)
        address_map.mapid = "geo-%s" % self.context.getId()

        return address_map

    @property
    def portrait_css_class(self):
        css_class = 'contactPortrait {}'
        gender = self.context.gender
        gender_css_class = 'gender-{}'.format(gender)
        return css_class.format(gender_css_class)

    @property
    def img_tag(self):
        return portrait_img_tag(self.context)

    def get_membership_url(self, memberblock):
        """
        Called from the template to construct the membership links so that
        the link points to the container of the memberblock and not the
        memberblock (obj) itself.
        """
        memberblock_container = aq_parent(aq_inner(memberblock))
        return memberblock_container.absolute_url() + '#' + memberblock.getId()


class ContactSummary(BrowserView):
    """Contactsummary view
    """

    def __call__(self):
        # Partial views are encoded wrong when not called by ajax in plone 5.
        # In consequece in tests, encoding errors can appear if we do not
        # disable the diazo theme 'encoder'.
        # Also see: https://github.com/4teamwork/ftw.testbrowser/commit/8ea19ffbefd251cd6712775336d2a1e76beb79de
        self.request.response.setHeader('X-Theme-Disabled', 'True')
        return super(ContactSummary, self).__call__()

    def get_review_state(self):
        return api.content.get_state(obj=self.context, default='')

    def safe_html(self, text):
        return safe_html(text)

    @property
    def img_tag(self):
        return portrait_img_tag(self.context)


if IS_PLONE_5:
    class RenameForm(PloneRenameForm):
        """ Plone's standard RenameForm will try to """

        description = _(
            u'description_rename_contact',
            default=u"Each item has a Short Name which you can change by " +
                    u"entering the new details below (Note that the 'Title' of " +
                    u"a Contact is generated from other fields, so it can't be " +
                    u"changed here)."
        )

        def updateWidgets(self):
            super(RenameForm, self).updateWidgets()
            self.widgets["new_title"].mode = HIDDEN_MODE



class MemberAccessor(object):
    """Generic member accessor adapter implementation for Dexterity content
       objects.
    """
    def __init__(self, context):
        object.__setattr__(self, 'context', context)
        object.__setattr__(self, 'contact', context.contact.to_object)

    def __getattr__(self, name):
        if self.context.acquire_address and hasattr(self.contact, name):
            return getattr(self.contact, name, None)
        return getattr(self.context, name, None)



from Acquisition import aq_inner
from Acquisition import aq_parent
from base64 import b64encode
from plone import api
from plone.memoize import instance
from plone.namedfile.utils import stream_data
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from StringIO import StringIO
from zc.relation.interfaces import ICatalog
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import unicodedata


ALPHABET = map(chr, range(ord('A'), ord('Z') + 1))
LETTERS = ALPHABET + ['#']


def encode(text):
    if not text:
        return ''
    if isinstance(text, unicode):
        return text.encode('utf-8')
    return text


def get_organization(context, portal_types=[]):
    """Traverse up to search a parent-object matching a portal_type
    If it reaches a portal_type, it returns its title.
    If it reaches the plone siteroot it returns an empty string
    """
    context = aq_inner(context)
    parent = aq_parent(context)
    while parent.portal_type not in portal_types:
        if IPloneSiteRoot.providedBy(parent):
            return ''
        parent = parent.aq_parent
    return parent.Title()


def safe_html(text):
    if not text:
        return ''

    if isinstance(text, unicode):
        text = text.encode('utf-8')

    ttool = api.portal.get_tool('portal_transforms')
    text = ttool.convertTo('text/x-html-safe', text).getData()

    # we need html br
    text = text.replace('\r\n', '<br />').replace('\n', '<br />')

    # we dont want p tags
    text = text.replace('<p>', '').replace('</p>', '')

    return text


def get_backreferences(source_object, from_interface):
    """
    Return back references from source object on specified attribute_name
    """
    catalog = getUtility(ICatalog)
    intids = getUtility(IIntIds)
    mtool = api.portal.get_tool('portal_membership')

    result = []

    for rel in catalog.findRelations({
            'to_id': intids.getId(aq_inner(source_object)),
            'from_interfaces_flattened': from_interface}):

        obj = intids.queryObject(rel.from_id)

        if obj is not None and mtool.checkPermission('View', obj):
            result.append(obj)
    return result


def get_contact_title(contact, display=None):
    """This function returns the contacttitle.

    The contact title is generated from the firstname and lastname
    or from the organization name. Depends on what fields are filled.
    """

    if contact.firstname and contact.lastname:
        if display == 'natural':
            return '%s %s' % (contact.firstname, contact.lastname)
        return '%s %s' % (contact.lastname, contact.firstname)
    return contact.organization or u"..."


def normalized_first_letter(text):
    """Normalizes the first letter of the text
    """
    text = make_sortable(text)
    return text[0].upper()


def make_sortable(text):
    """Makes the text sortable
    """
    text = text.lower()
    text = text.decode('utf-8')
    normalized = unicodedata.normalize('NFKD', text)
    text = u''.join(
        [c for c in normalized if not unicodedata.combining(c)])
    text = text.encode('utf-8')
    return text


class AlphabeticLetters(object):
    """Provides a function to generate letters with different attributes
    """
    def letters(self, current_letter, brains):
        """Returns a dict with all available letters and
        aditional infos for each letter.

        The dict has:

        label: the letter name
        has_contents: css-class if there are objs with the letter
        current: css-class if the letter is the current letter
        """
        letters_with_content = self._letters_with_content(brains)

        for letter in LETTERS:
            has_contents = letter in letters_with_content and \
                "withContent" or ''
            current = letter == current_letter and 'active' or ''
            yield {'label': letter,
                   'has_contents': has_contents,
                   'current': current}

    @instance.memoize
    def _letters_with_content(self, brains):
        letters = set([])
        for contactname in [brain.Title for brain in brains]:
            first_letter = normalized_first_letter(contactname)
            if first_letter in ALPHABET:
                letters.add(first_letter)
            else:
                letters.add('#')

        return sorted(letters, key=LETTERS.index)


def generate_vcard(contact):
    # TODO: Refactoring
    io = StringIO()

    gender_map = {
        'm': 'M',
        'f': 'F',
        '': 'U'
    }

    def addProp(name, value):
        if not value:
            return
        io.write('{0}:{1}\n'.format(encode(name), encode(value)))

    addProp('BEGIN', 'VCARD')
    addProp('VERSION', '3.0')
    addProp('N', '{0};{1};;{2}'.format(encode(contact.lastname),
                                       encode(contact.firstname),
                                       encode(contact.salutation)))
    addProp('FN', encode(contact.title))
    addProp('GENDER', gender_map.get(encode(contact.gender), ''))
    addProp('ORG', encode(contact.organization))
    addProp('ADR;TYPE=WORK', ';;{0};{1};;{2};{3}'.format(
        encode(contact.address.replace('\n', '\\n').replace('\r', '')),
        encode(contact.city),
        encode(contact.postal_code),
        encode(contact.country)))
    addProp('EMAIL', encode(contact.email))
    addProp('TEL;TYPE=WORK', encode(contact.phone_office))
    addProp('TEL;TYPE=WORK;TYPE=CELL', encode(contact.phone_mobile))
    addProp('TEL;TYPE=WORK;TYPE=FAX', encode(contact.fax))
    addProp('URL', encode(contact.www))
    addProp('ROLE', encode(contact.function))
    addProp('TEL;TYPE=HOME', encode(contact.phone_private))
    addProp('ADR;TYPE=HOME', ';;{0};{1};;{2};'.format(
        encode(contact.address_private.replace(
            '\r\n', '\\n').replace('\r', '')),
        encode(contact.city_private),
        encode(contact.postal_code_private)))

    if contact.image:
        imgdata = StringIO()
        imgdata.write(stream_data(contact.image))
        addProp('PHOTO;ENCODING=B', b64encode(imgdata.getvalue()))

    addProp('END', 'VCARD')
    io.seek(0)

    return io

def portrait_img_tag(contact):
    scaler = contact.restrictedTraverse('@@images')
    return scaler.scale('image', scale='portrait', direction='down').tag()



from __future__ import absolute_import
from ftw.contacts.interfaces import ILDAPSearch, IContactsSettings
from ldap.controls import SimplePagedResultsControl
from plone import api
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import base_hasattr
from zope.interface import implements
from zope.site.hooks import getSite
import ldap
import logging

try:
    # python-ldap 2.4
    LDAP_CONTROL_PAGED_RESULTS = ldap.CONTROL_PAGEDRESULTS
    PYTHON_LDAP_24 = True
except AttributeError:
    # python-ldap 2.3
    LDAP_CONTROL_PAGED_RESULTS = ldap.LDAP_CONTROL_PAGE_OID
    PYTHON_LDAP_24 = False

logger = logging.getLogger('ftw.contacts.sync')


class LDAPSearch(object):
    """Utility for searching in LDAP.

       Uses connection settings defined in LDAPUserFolder.
    """
    implements(ILDAPSearch)

    def connect(self, plugin_id):
        ldap_uf = self.ldap_userfolder(plugin_id)
        if ldap_uf is None:
            return None
        self.base_dn = ldap_uf.users_base
        return ldap_uf._delegate.connect()

    def ldap_userfolder(self, plugin_id):
        site = getSite()
        if site is None:
            return None

        uf = getToolByName(site, 'acl_users')

        # No plugin id configured, try to find an ldap plugin
        if not plugin_id:
            for obj in uf.objectValues():
                if base_hasattr(obj, '_getLDAPUserFolder'):
                    return obj._getLDAPUserFolder()
            return None

        # Get ldap plugin configured in registry
        ldap_plugin = uf.unrestrictedTraverse(plugin_id.encode('ascii'), None)
        if ldap_plugin is None:
            return None
        return ldap_plugin._getLDAPUserFolder()

    def search(self, plugin_id=None, base_dn=None, scope=None,
               filter='(objectClass=*)', attrs=[]):
        scope = getattr(ldap, scope) if scope else ldap.SCOPE_SUBTREE
        conn = self.connect(plugin_id)
        if conn is None:
            return []

        if base_dn is None:
            base_dn = self.base_dn

        # Get paged results to prevent exceeding server size limit
        page_size = 1000
        if PYTHON_LDAP_24:
            lc = SimplePagedResultsControl(size=page_size, cookie='')
        else:
            lc = SimplePagedResultsControl(LDAP_CONTROL_PAGED_RESULTS,
                                           True,
                                           (page_size, ''),)
        is_last_page = False
        results = []
        while not is_last_page:
            msgid = conn.search_ext(base_dn,
                                    scope,
                                    filter,
                                    attrs,
                                    serverctrls=[lc])
            rtype, rdata, rmsgid, serverctrls = conn.result3(msgid)
            results.extend(rdata)
            pctrls = [c for c in serverctrls
                      if c.controlType == LDAP_CONTROL_PAGED_RESULTS]
            if pctrls:
                if PYTHON_LDAP_24:
                    cookie = pctrls[0].cookie
                    if cookie:
                        lc.cookie = cookie
                    else:
                        is_last_page = True
                else:
                    cookie = pctrls[0].controlValue[1]
                    if cookie:
                        lc.controlValue[1] = cookie
                    else:
                        is_last_page = True
            else:
                is_last_page = True
                logger.warn("Server ignores paged results control (RFC 2696).")
        return results



from ftw.contacts.utils import encode
from ftw.contacts.utils import generate_vcard
from Products.Five.browser import BrowserView


class DownloadVCardView(BrowserView):
    """Download vCard of contact
    """
    def __call__(self):
        response = self.request.response
        response.setHeader("Content-type", "text/vcard")
        filename = encode('%s.vcf' % self.context.Title())
        response.setHeader("Content-Disposition",
                           'inline; filename="%s"' % filename)
        return generate_vcard(self.context).getvalue()



from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.contacts.utils import AlphabeticLetters
from Products.CMFCore.utils import getToolByName
from unittest import TestCase


class TestAlphabeticalSubjectListing(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.catalog = getToolByName(self.portal, 'portal_catalog')

    def test_mark_current_letter(self):
        brains = self.catalog(portal_type="ftw.contacts.Contact")
        letters = AlphabeticLetters().letters('I', brains)
        letter = [
            letter.get('label') for letter in letters
            if letter.get('current')
            ][0]

        self.assertEqual('I', letter)

    def test_mark_no_letter(self):
        brains = self.catalog(portal_type="ftw.contacts.Contact")
        letters = AlphabeticLetters().letters('', brains)
        letter = [
            letter.get('label') for letter in letters
            if letter.get('current')
            ]

        self.assertEqual([], letter)

    def test_mark_special_letter_as_current_letter(self):
        brains = self.catalog(portal_type="ftw.contacts.Contact")
        letters = AlphabeticLetters().letters('#', brains)
        letter = [
            letter.get('label') for letter in letters
            if letter.get('current')
            ][0]

        self.assertEqual('#', letter)

    def test_mark_letters_with_contents(self):
        contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))

        create(Builder('contact')
               .with_maximal_info(u'Chuck', u'4orris')
               .within(contactfolder))

        create(Builder('contact')
               .with_maximal_info(u'J\xe4mes', u'B\xf6nd')
               .within(contactfolder))

        brains = self.catalog(portal_type="ftw.contacts.Contact")
        letters = AlphabeticLetters().letters('I', brains)

        letter = [
            letter.get('label') for letter in letters
            if letter.get('has_contents')
            ]

        self.assertEqual(['B', '#'], letter)



from ftw.upgrade import UpgradeStep


class InstallRefwidget(UpgradeStep):
    """Install the reference widget from "ftw.referencewidget"
    """

    def __call__(self):
        self.setup_install_profile('profile-ftw.referencewidget:default')



from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.contacts.utils import get_contact_title
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest import TestCase


class TestContactTitle(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_returns_placeholder_if_no_first_last_and_org(self):
        contact = create(Builder('contact'))
        self.assertEqual(u"...", get_contact_title(contact))

    def test_returns_org_if_no_first_name(self):
        contact = create(Builder('contact').having(
            organization=u"CIA",
            lastname=u"Norris"))
        self.assertEqual(u"CIA", get_contact_title(contact))

    def test_returns_org_if_no_last_name(self):
        contact = create(Builder('contact').having(
            organization=u"CIA",
            firstname=u"Chuck"))
        self.assertEqual(u"CIA", get_contact_title(contact))

    def test_returns_org_if_no_first_and_last_name(self):
        contact = create(Builder('contact').having(
            organization=u"CIA"))
        self.assertEqual("CIA", get_contact_title(contact))

    def test_returns_last_and_first_name_if_exists_without_org(self):
        contact = create(Builder('contact').having(
            firstname=u"Chuck",
            lastname=u"Norris"))
        self.assertEqual("Norris Chuck", get_contact_title(contact))

    def test_returns_last_and_first_name_if_exists_with_org(self):
        contact = create(Builder('contact').having(
            firstname=u"Chuck",
            lastname=u"N\xf6rris",
            organization=u"CIA"))
        self.assertEqual(u"N\xf6rris Chuck", get_contact_title(contact))

    def test_returns_first_and_last_name_if_exists_with_format_natural(self):
        contact = create(Builder('contact').having(
            firstname=u"Chuck",
            lastname=u"N\xf6rris"))
        self.assertEqual(
            u"Chuck N\xf6rris",
            get_contact_title(contact, display="natural"))



from ftw.contacts.interfaces import IMemberAccessor
from ftw.contacts.utils import safe_html
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView


class MemberView(BrowserView):
    """View to display a member
    """
    def __init__(self, context, request):
        super(MemberView, self).__init__(context, request)
        self.contact = self.get_related_contact()

    def memberaccessor(self):
        if not self.contact:
            return None
        return IMemberAccessor(self.context)

    @property
    def has_permission(self):
        mtool = getToolByName(self.context, "portal_membership")
        return bool(
            mtool.checkPermission("Modify portal content", self.context))

    def get_related_contact(self):
        if self.context.contact:
            return self.context.contact.to_object
        return None

    @property
    def has_related_contact(self):
        return bool(self.contact)

    def safe_html(self, text):
        return safe_html(text)

    @property
    def function(self):
        # The function of the member block has precedes the function of the contact.
        if self.context.function:
            return self.context.function
        if self.context.acquire_address and self.contact:
            return self.contact.function
        return ''



from ftw.upgrade import UpgradeStep


class DefineScaleForContactSummaryImage(UpgradeStep):
    """Define scale for contact summary image.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.contacts.utils import make_sortable
from unittest import TestCase


class TestMakeSortable(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def test_text_is_lowercased(self):
        self.assertEquals('foo',
                          make_sortable('FoO'))

    def test_umlauts_are_normalized(self):
        self.assertEquals('loffel',
                          make_sortable('L\xc3\xb6ffel'))

    def test_cedillas_are_removed(self):
        self.assertEquals('francais',
                          make_sortable('fran\xc3\xa7ais'))



from ftw.contacts import _
from ftw.contacts.interfaces import IContactFolder
from plone.dexterity.content import Container
from plone.supermodel import model
from zope import schema
from zope.interface import implements


class IContactFolderSchema(model.Schema):
    """A contactfolder type schema interface
    """

    hide_contacts_image = schema.Bool(
        title=_(u'label_contactdirectory_hide_contacts_image',
                default=u'Hide images of contacts'),
        default=False,
        description=_(u'help_contactdirectory_hide_contacts_image',
                      default=u'If activated, the images of the contacts are not shown in the listing-view.')
    )


class ContactFolder(Container):
    """A contactfolder
    """
    implements(IContactFolder)



from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.api.content import delete
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest import TestCase

import transaction


class TestMemberBlockView(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))

    @browsing
    def test_call_view_does_not_fail(self, browser):
        contact = create(Builder('contact')
                         .with_minimal_info(u'Ch\xf6ck', u'4orris')
                         .within(self.contactfolder))

        create(Builder('member block')
               .within(self.portal)
               .contact(contact)
               .titled(u"A MemberBlock")
               .having(show_title=True))

        browser.login().visit(self.portal, view='simplelayout-view')
        self.assertEqual(u'A MemberBlock', browser.css('.ftw-contacts-memberblock h3').first.text)

    @browsing
    def test_block_with_deleted_contact_returns_hint_for_editors(self, browser):

        contact = create(Builder('contact')
                         .with_minimal_info(u'Ch\xf6ck', u'4orris')
                         .within(self.contactfolder))

        member_block = create(Builder('member block')
                              .within(self.contactfolder)
                              .contact(contact))

        delete(contact, check_linkintegrity=False)
        transaction.commit()

        browser.login().visit(member_block, view="block_view")

        self.assertEqual(1, len(browser.css('#member-no-contact-exist')))

    @browsing
    def test_block_without_contact_renders_hint_for_editors(self, browser):
        page = create(Builder('sl content page'))
        contact = create(Builder('contact')
                         .with_minimal_info(u'Ch\xf6ck', u'4orris')
                         .within(self.contactfolder))
        member_block = create(Builder('member block').within(page).contact(contact))

        # Remove the referenced contact.
        member_block.contact = None
        transaction.commit()

        browser.raise_http_errors = False
        browser.login().visit(member_block, view='block_view')
        self.assertEqual(
            [
                'The related contact does no longer exists. '
                'Delete this content or edit it to replace the deleted with an existing contact'
            ],
            browser.css('#member-no-contact-exist').text,
            'A hint should be rendered about the block\'s missing contact.'
        )

    @browsing
    def test_call_returns_member_view(self, browser):
        contact = create(Builder('contact')
                         .with_maximal_info(u'Ch\xf6ck', u'4orris')
                         .within(self.contactfolder))

        create(Builder('member block')
               .within(self.portal)
               .contact(contact))

        browser.login().visit(self.portal, view='simplelayout-view')

        self.assertEqual(0, len(browser.css('.ftw-contacts-memberblock #member-no-contact-exist')))
        self.assertEqual(0, len(browser.css('.ftw-contacts-memberblock #member-empty')))
        self.assertEqual(1, len(browser.css('.ftw-contacts-memberblock .memberContactInfo')))

    @browsing
    def test_block_with_deleted_contact_returns_hint_for_anonymous(self, browser):
        contact = create(Builder('contact')
                         .with_minimal_info(u'Ch\xf6ck', u'4orris')
                         .within(self.contactfolder))

        member_block = create(Builder('member block')
                              .within(self.contactfolder)
                              .contact(contact))

        delete(contact, check_linkintegrity=False)
        transaction.commit()

        browser.visit(member_block, view="block_view")

        self.assertEqual(1, len(browser.css('#member-empty')))

    @browsing
    def test_block_without_contact_renders_hint_for_anonymous(self, browser):
        page = create(Builder('sl content page'))
        contact = create(Builder('contact')
                         .with_minimal_info(u'Ch\xf6ck', u'4orris')
                         .within(self.contactfolder))
        member_block = create(Builder('member block').within(page).contact(contact))

        # Remove the referenced contact.
        member_block.contact = None
        transaction.commit()

        browser.raise_http_errors = False
        browser.visit(member_block, view='block_view')
        self.assertEqual(
            [],
            browser.css('#member-no-contact-exist').text,
            'Blocks without contact should render nothing for anonymous users.'
        )

    @browsing
    def test_function_is_acquired(self, browser):
        """
        This test makes sure that the function of the contact is rendered
        on the member block if the address is acquired.
        """
        contact = create(Builder('contact')
                         .with_minimal_info(u'Dent', u'Arthur')
                         .having(address=u'End Of The Universe')
                         .having(function=u'Function of the contact')
                         .within(self.contactfolder))

        member_block = create(Builder('member block')
                              .within(self.contactfolder)
                              .having(acquire_address=True)
                              .contact(contact))

        browser.visit(member_block, view='block_view')
        self.assertEqual(
            ['Dent Arthur Function of the contact'],
            browser.css('.memberContactInfo').text
        )

    @browsing
    def test_memberblock_function_takes_precendence_over_contact_function(self, browser):
        """
        This test makes sure that the function of the memberblock
        takes precedence over the acquired function from the contact.
        """
        contact = create(Builder('contact')
                         .with_minimal_info(u'Dent', u'Arthur')
                         .having(address=u'End Of The Universe')
                         .having(function=u'Function of the contact')
                         .within(self.contactfolder))

        member_block = create(Builder('member block')
                              .within(self.contactfolder)
                              .having(acquire_address=True)
                              .having(function=u'Function of the member block')
                              .contact(contact))

        browser.visit(member_block, view='block_view')
        self.assertEqual(
            ['Dent Arthur Function of the member block'],
            browser.css('.memberContactInfo').text
        )



from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.tests import FunctionalTestCase
from ftw.testbrowser import browsing


class TestHideContactImages(FunctionalTestCase):

    def setUp(self):
        super(TestHideContactImages, self).setUp()
        self.grant('Manager')

    @browsing
    def test_contact_images_are_hidden_when_hide_images_checkbox_is_active(self, browser):
        contactfolder = create(Builder('contact folder')
                               .titled(u'Contact folder')
                               .having(hide_contacts_image=True))

        contact = create(Builder('contact')
                         .with_maximal_info(u'Chuck', u'Norris')
                         .within(contactfolder))

        browser.login().visit(contact, view='contact_summary')

        self.assertEqual(1, len(browser.css('div.contactSummary')))
        self.assertEqual(0, len(browser.css('a.contactImage')))

    @browsing
    def test_contact_images_are_shown_when_hide_images_checkbox_is_deactivated(self, browser):
        contactfolder = create(Builder('contact folder')
                               .titled(u'Contact folder')
                               .having(hide_contacts_image=False))

        contact = create(Builder('contact')
                         .with_maximal_info(u'Chuck', u'Norris')
                         .within(contactfolder))

        browser.login().visit(contact, view='contact_summary')

        self.assertEqual(1, len(browser.css('a.contactImage')))



from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.contents.contactfolder import IContactFolderSchema
from ftw.contacts.interfaces import IContactFolder
from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from unittest import TestCase
from zope.component import createObject
from zope.component import queryUtility


class TestContactFolderInstallation(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_adding(self):
        contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))

        self.assertTrue(IContactFolder.providedBy(contactfolder))

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name='ftw.contacts.ContactFolder')
        self.assertNotEquals(None, fti)

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name='ftw.contacts.ContactFolder')
        schema = fti.lookupSchema()
        self.assertEqual(IContactFolderSchema, schema)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name='ftw.contacts.ContactFolder')
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(IContactFolderSchema.providedBy(new_object))



from zope.interface import Interface
from zope import schema


class IMemberRegistry(Interface):
    member_nav_tree_query_path = schema.TextLine(
        title=u"Member navigation tree query path",
        description=u"Add the path to your contactdirectory: '/address'",
        default=u"",
        required=False)
